@@ -566,6 +566,10 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "Content type for the video stream", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("playing", "playing",
+				     "Whether the track is playing", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
@@ -1314,6 +1318,60 @@ emit_entry_parsed_signal (EntryParsedSignalData *data)
 	return FALSE;
 }
 
+gboolean
+totem_pl_parser_fix_string (const char  *name,
+			    const char  *value,
+			    char       **ret)
+{
+	char *fixed = NULL;
+
+	/* Check for UTF-8 or ISO8859-1 string */
+	if (g_utf8_validate (value, -1, NULL) == FALSE) {
+		fixed = g_convert (value, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+		if (fixed == NULL) {
+			g_warning ("Ignored non-UTF-8 and non-ISO8859-1 string for field '%s'", name);
+			return FALSE;
+		}
+	}
+
+	/* Remove trailing spaces from titles */
+	if (g_str_equal (name, TOTEM_PL_PARSER_FIELD_TITLE)) {
+		if (fixed == NULL)
+			fixed = g_strchomp (g_strdup (value));
+		else
+			g_strchomp (fixed);
+	}
+
+	*ret = fixed;
+
+	return TRUE;
+}
+
+void
+totem_pl_parser_add_hash_table (TotemPlParser *parser,
+				GHashTable    *metadata,
+				const char    *uri,
+				gboolean       is_playlist)
+{
+	if (g_hash_table_size (metadata) > 0 || uri != NULL) {
+		EntryParsedSignalData *data;
+
+		/* Make sure to emit the signals asynchronously, as we could be in the main loop
+		 * *or* a worker thread at this point. */
+		data = g_new (EntryParsedSignalData, 1);
+		data->parser = g_object_ref (parser);
+		data->uri = g_strdup (uri);
+		data->metadata = g_hash_table_ref (metadata);
+
+		if (is_playlist == FALSE)
+			data->signal_id = totem_pl_parser_table_signals[ENTRY_PARSED];
+		else
+			data->signal_id = totem_pl_parser_table_signals[PLAYLIST_STARTED];
+
+		CALL_ASYNC (parser, emit_entry_parsed_signal, data);
+	}
+}
+
 static void
 totem_pl_parser_add_uri_valist (TotemPlParser *parser,
 				const gchar *first_property_name,
@@ -1394,21 +1452,10 @@ totem_pl_parser_add_uri_valist (TotemPlParser *parser,
 		if (string != NULL && string[0] != '\0') {
 			char *fixed = NULL;
 
-			if (g_utf8_validate (string, -1, NULL) == FALSE) {
-				fixed = g_convert (string, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
-				if (fixed == NULL) {
-					g_value_unset (&value);
-					name = va_arg (var_args, char*);
-					continue;
-				}
-			}
-
-			/* Remove trailing spaces from titles */
-			if (strcmp (name, "title") == 0) {
-				if (fixed == NULL)
-					fixed = g_strchomp (g_strdup (string));
-				else
-					g_strchomp (fixed);
+			if (!totem_pl_parser_fix_string (name, string, &fixed)) {
+				g_value_unset (&value);
+				name = va_arg (var_args, char*);
+				continue;
 			}
 
 			/* Add other values to the metadata hashtable */
@@ -1425,23 +1472,10 @@ totem_pl_parser_add_uri_valist (TotemPlParser *parser,
 		//FIXME fix this! 396710
 	}
 
-	if (g_hash_table_size (metadata) > 0 || uri != NULL) {
-		EntryParsedSignalData *data;
-
-		/* Make sure to emit the signals asynchronously, as we could be in the main loop
-		 * *or* a worker thread at this point. */
-		data = g_new (EntryParsedSignalData, 1);
-		data->parser = g_object_ref (parser);
-		data->uri = g_strdup (uri);
-		data->metadata = g_hash_table_ref (metadata);
-
-		if (is_playlist == FALSE)
-			data->signal_id = totem_pl_parser_table_signals[ENTRY_PARSED];
-		else
-			data->signal_id = totem_pl_parser_table_signals[PLAYLIST_STARTED];
-
-		CALL_ASYNC (parser, emit_entry_parsed_signal, data);
-	}
+	totem_pl_parser_add_hash_table (parser,
+					metadata,
+					uri,
+					is_playlist);
 
 	g_hash_table_unref (metadata);
 
@@ -1835,8 +1869,10 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 
 	/* Not a directory on http though */
 	if (g_strcmp0 (mimetype, "inode/directory") == 0 &&
-	    g_file_has_uri_scheme (file, "http"))
-		g_clear_pointer (&mimetype, g_free);
+	    g_file_has_uri_scheme (file, "http")) {
+		g_free (mimetype);
+		mimetype = NULL;
+	}
 
 	DEBUG(file, g_print ("_get_mime_type_for_name for '%s' returned '%s'\n", uri, mimetype));
 	if (mimetype == NULL || strcmp (UNKNOWN_TYPE, mimetype) == 0
@@ -2421,6 +2457,7 @@ totem_pl_parser_can_parse_from_data (const char *data,
 	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
 		if (strcmp (special_types[i].mimetype, mimetype) == 0) {
 			D(g_message ("Is special type '%s'", mimetype));
+			g_free (mimetype);
 			return TRUE;
 		}
 	}
@@ -2432,14 +2469,18 @@ totem_pl_parser_can_parse_from_data (const char *data,
 				gboolean retval = ((* dual_types[i].iden) (data, len) != NULL);
 				D(g_message ("%s dual type '%s'",
 					     retval ? "Is" : "Is not", mimetype));
+				g_free (mimetype);
 				return retval;
 			}
+			g_free (mimetype);
 			return FALSE;
 		}
 	}
 
 	D(g_message ("Is unsupported mime-type '%s'", mimetype));
 
+	g_free (mimetype);
+
 	return FALSE;
 }
 
